@@ -33,7 +33,7 @@ class TestHTTPClient < Test::Unit::TestCase
     @client.get(serverurl)
     lines = str.split(/(?:\r?\n)+/)
     assert_equal("= Request", lines[0])
-    assert_match(/^User-Agent: agent_name_foo/, lines[4])
+    assert_match(/^User-Agent: agent_name_foo \(#{HTTPClient::VERSION}/, lines[4])
   end
 
   def test_from
@@ -591,6 +591,16 @@ EOS
     assert_nil(res.content)
   end
 
+  def test_get_with_block_chunk_string_recycle
+    @client.read_block_size = 2
+    body = []
+    res = @client.get(serverurl + 'servlet') { |str|
+      body << str
+    }
+    assert_equal(2, body.size)
+    assert_equal("get", body.join) # Was "tt" by String object recycle...
+  end
+
   def test_post
     assert_equal("post", @client.post(serverurl + 'servlet').content[0, 4])
     res = @client.post(serverurl + 'servlet', {1=>2, 3=>4})
@@ -1273,24 +1283,24 @@ EOS
       end
     }
     # allocate 10 keep-alive connections
-    10.times.to_enum.map {
+    (0...10).to_a.map {
       Thread.new {
         assert_equal("12345", client.get(endpoint).content)
       }
-    }.each(&:join)
+    }.each { |th| th.join }
     # send 5 requests, which should get KeepAliveDesconnected.
     # doing these requests, rest keep-alive connections are invalidated.
-    5.times.to_enum.map {
+    (0...5).to_a.map {
       Thread.new {
         assert_equal("23456", client.get(endpoint).content)
       }
-    }.each(&:join)
+    }.each { |th| th.join }
     # rest requests won't get KeepAliveDisconnected; how can I check this?
-    10.times.to_enum.map {
+    (0...10).to_a.map {
       Thread.new {
         assert_equal("34567", client.get(endpoint).content)
       }
-    }.each(&:join)
+    }.each { |th| th.join }
   end
 
   def create_keepalive_thread(count, sock)
@@ -1356,6 +1366,19 @@ EOS
     end
   end
 
+  def test_body_param_order
+    ary = ('b'..'d').map { |k| ['key2', k] } << ['key1', 'a'] << ['key3', 'z']
+    assert_equal("key2=b&key2=c&key2=d&key1=a&key3=z", HTTP::Message.escape_query(ary))
+  end
+
+  if RUBY_VERSION > "1.9"
+    def test_charset
+      body = @client.get(serverurl + 'charset').body
+      assert_equal(Encoding::EUC_JP, body.encoding)
+      assert_equal('あいうえお'.encode(Encoding::EUC_JP), body)
+    end
+  end
+
 private
 
   def check_query_get(query)
@@ -1379,7 +1402,7 @@ private
       :DocumentRoot => File.dirname(File.expand_path(__FILE__))
     )
     @serverport = @server.config[:Port]
-    [:hello, :sleep, :servlet_redirect, :redirect1, :redirect2, :redirect3, :redirect_self, :relative_redirect, :chunked, :largebody, :status, :compressed].each do |sym|
+    [:hello, :sleep, :servlet_redirect, :redirect1, :redirect2, :redirect3, :redirect_self, :relative_redirect, :chunked, :largebody, :status, :compressed, :charset].each do |sym|
       @server.mount(
 	"/#{sym}",
 	WEBrick::HTTPServlet::ProcHandler.new(method("do_#{sym}").to_proc)
@@ -1456,6 +1479,7 @@ private
   end
 
   def do_compressed(req, res)
+    res['content-type'] = 'application/octet-stream'
     if req.query['enc'] == 'gzip'
       res['content-encoding'] = 'gzip'
       res.body = GZIP_CONTENT
@@ -1465,6 +1489,15 @@ private
     end
   end
 
+  def do_charset(req, res)
+    if RUBY_VERSION > "1.9"
+      res.body = 'あいうえお'.encode("euc-jp")
+      res['Content-Type'] = 'text/plain; charset=euc-jp'
+    else
+      res.body = 'this endpoint is for 1.9 or later'
+    end
+  end
+
   def do_status(req, res)
     res.status = req.query['status'].to_i
   end
@@ -1485,6 +1518,7 @@ private
     end
 
     def do_POST(req, res)
+      res["content-type"] = "text/plain" # iso-8859-1, not US-ASCII
       res.body = 'post,' + req.body.to_s
       res["x-query"] = body_response(req)
     end
